/*
poisz.c Version 1.4.0 - Calculate sample size for a
poisson distribution based on:
   confidence level
   error of estimation
   population mean
Copyright (C) 2005  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* This test is based on: */
/* Lincoln L. Chao */
/* Statistics for Management */
/* Palo Alto, CA: The Scientific Press, 1984 */
/* Chapter 9, Statistical Estimation */
/* Section 4, Determination of Sample Size */

/* Lincoln L. Chao was at California State University, */
/* Long Beach, CA, while writing this book. */

/* Sample test: */

/* poisz 95.0 0.01 2.0 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s confidence "
      "error_rate mu\n",
      pgm);
   fprintf(stderr,"Where confidence is "
      "90.0, 95.0, 98.0, 99.0\n");
   fprintf(stderr,"      error_rate is 0.001 to 0.1\n");
   fprintf(stderr,"      mu is 0.1 to 30.0\n");
   fprintf(stderr,"Example: %s "
      "95.0 0.02 2.0\n",
      pgm);
   fprintf(stderr,"mu is the population "
      "mean for the poisson distribution\n");
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int size;
   double confidence;
   double errrate;
   double mu;
   int getn(double confidence, double errrate,
      double psuccess);
   if (argc != 4) putstx(*argv);
   confidence = atof(*(argv+1));
   if (confidence == 90.0 || confidence == 95.0
      || confidence == 98.0 || confidence == 99.0);
   else
      {
      fprintf(stderr,"Invalid confidence level %s.\n",
	 *(argv+1));
      putstx(*argv);
      } /* bad confidence */
   errrate = atof(*(argv+2));
   if (errrate < 0.001 || errrate > 0.1)
      {
      fprintf(stderr,"Invalid error rate %s.\n",
	 *(argv+2));
      putstx(*argv);
      } /* bad error rate */
   mu = (double) atof(*(argv+3));
   if (mu < 0.1)
      {
      fprintf(stderr,"Population mean (mu) "
	 "is too small.\n");
      putstx(*argv);
      } /* mu too small */
   if (mu > 30.0)
      {
      fprintf(stderr,"Population mean (mu) "
	 "is too large.\n");
      putstx(*argv);
      } /* if mu too large */
   /* probability of success = mu / 100.0 */
   size = getn(confidence, errrate, mu / 100.0);
   printf("Sample size is %d\n", size);
   return(0);
   } /* main */
