/*
rnd2.c - random number generator based on 16 byte seed
Version 1.4.0
Copyright (C) 1999-2005  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Usage:
   	void rnd(unsigned char *sd);
*/

/*
Called by:
	unsigned int rndpwr(int bits,
		unsigned char *sd);
*/

/*
initialize the seed with:
   	unsigned char *sd;
   	sd = (unsigned char *) rndinit();
*/

/*
rnd2.c replaces rnd.s for non-Intel platforms
To compile on non-Intel platforms:

	make -f nonintel.mak

rnd2.c is 20x slower than rnd.s
Run parallel test between Intel and non-Intel
platforms using same seed.  Verify that generated
data is identical.
*/

#define SEEDLEN 16
#define HALFLEN 8
#define BYTESIZE 8

int rnd(sd)
unsigned char *sd;
   {
   int i,j;
   int rslt;
   unsigned char sd2[SEEDLEN+16];
   unsigned char *p,*q,*r;
   p = (unsigned char *) sd;
   q = (unsigned char *) sd2;
   r = (unsigned char *) p + SEEDLEN;
   j = 0;
   while (p < r)
      {
      i = (*p++ << 1) | j;
      j = (i >> BYTESIZE) & 1;
      *q++ = (unsigned char) i & 255;
      }
   p = (unsigned char *) sd + HALFLEN;
   q = (unsigned char *) sd;
   r = (unsigned char *) p + HALFLEN;
   j = 0;
   while (p < r)
      {
      i = *p + *q++ + j;
      j = (i >> BYTESIZE) & 1;
      *p++ = (unsigned char) i & 255;
      }
   p = (unsigned char *) sd;
   q = (unsigned char *) sd2;
   r = (unsigned char *) p + SEEDLEN;
   j = 0;
   while (p < r)
      {
      i = *p + *q++ + j;
      j = (i >> BYTESIZE) & 1;
      *p++ = (unsigned char) i & 255;
      }
   p--;
   rslt = (int) (*p >> 7) & 1;
   return(rslt);
   } /* rnd */
